//
//  DLPObjectContext.h
//  DLSubstrate
//
//  Created by Michael Clark on 11-03-07.
//  Copyright 2011 Marketcircle, Inc. All rights reserved.
//

#import "MCPObjectContext.h"

#import "GWSubstrateDefines.h"

@class GWUser, DLOptionsCache;
@class DLCriteriaRegistry;
@class MCAlertManager;
@class GWReminderManager;
@class DLTimeblockOccurrenceCache;
@class MCPFetchResultsCache;
@class MCPDataStoreInfo;
@class GWDatabaseConfiguration;
@class DLLocalNotificationManager;
@class MCPAbstractDatabaseConnection;
@class DLCalendarUserSyncManager;
@class DLFieldOptionsManager;

@interface DLPObjectContext : MCPObjectContext {
    
    GWUser *user;
    DLOptionsCache *optionsCache;
    MCPFetchResultsCache *fetchResultsCache;
#if TARGET_OS_IPHONE
    DLLocalNotificationManager *localNotifManager;
#else
    DLCriteriaRegistry *criteriaRegistry;
    GWReminderManager* reminderManager;
#endif
    
    MCAlertManager*   alertManager;
    
    DLTimeblockOccurrenceCache *timeblockOccurrenceCache;
    
    BOOL isLinkingInBulk;
    
    NSInteger autoLinkContactNoteToOrg;  		// 0 == not checked, 1 == YES, 2 == NO;
    NSInteger autoLinkContactTaskToOrg;  		// 0 == not checked, 1 == YES, 2 == NO;
    NSInteger autoLinkContactTimeblockToOrg;  // 0 == not checked, 1 == YES, 2 == NO;
    
    BOOL areNotificationsEnabled; // useful when doing bulk stuff
    
    id reportLocator; // the locator tied to this database
    
    NSInteger accessdPort;
    
}
@property(strong) GWUser *user;

// Registries and Managers
@property(nonatomic,readonly,strong) DLOptionsCache *optionsCache;
@property(nonatomic,readonly,strong) DLFieldOptionsManager *fieldOptionsManager;

@property(nonatomic,readonly,strong) MCPFetchResultsCache *fetchResultsCache;
#if TARGET_OS_IPHONE
@property(nonatomic,readonly,strong) DLLocalNotificationManager *localNotifManager;
#else
@property(nonatomic,readonly,strong) DLCriteriaRegistry *criteriaRegistry;
@property(nonatomic,readonly,strong) GWReminderManager* reminderManager;
#endif
@property(nonatomic,readonly,strong) MCAlertManager* alertManager;
@property(nonatomic,strong) DLTimeblockOccurrenceCache *timeblockOccurrenceCache;

@property(nonatomic,readwrite,assign) BOOL isLinkingInBulk;

@property(nonatomic,strong) id reportLocator;

@property(nonatomic,readwrite,assign) BOOL currentUserUsesSyncList;

@property(nonatomic,assign) BOOL shouldAutoLinkContactNotesAndEmailsToDefaultOrg;
@property(nonatomic,assign) BOOL shouldAutoLinkContactTimeblocksToDefaultOrg;
@property(nonatomic,assign) BOOL shouldAutoLinkContactTasksToDefaultOrg;
@property(nonatomic,assign) BOOL shouldAutoLinkContactObjectivesToDefaultOrg;

@property(nonatomic,assign) BOOL permissionsSystemPreference; // Returns a preference in the database that tells us whether the permissions sytem should be on or not

// DLF-4162 - stuff higher up can look at this flag to determine whether or not to save this object context (most likely a sub-object context) on logout or not. in some cases (i.e. dupe checker) we do not want changes in sub-object contexts saved because we're using the oc to present a preview to the user, it should not be committed without user action. Defaults to YES - Ali
@property(nonatomic,assign) BOOL autosaveOnLogout;

#if ! TARGET_OS_IPHONE
@property(nonatomic,strong) DLCalendarUserSyncManager* calendarSyncManager;
#endif


#if TARGET_OS_IPHONE
+ (NSNumber *)currentUserID;
#endif

+ (DLPObjectContext *)objectContextForDataStoreInfo:(MCPDataStoreInfo *)aDataStoreInfo username:(NSString *)aUsername error:(NSError**)error;
- (BOOL)configureWithDayliteUsername: (NSString*)aUsername error: (NSError**)error;

+ (DLPObjectContext*)objectContextWithSuperUserForDataStoreInfo: (MCPDataStoreInfo*)aDataStoreInfo error: (NSError**)error;

#pragma mark Configuration
- (void)setupAlertManager;
- (void)setupReminderManager;

#pragma mark DatabasePreference
- (void)setDatabasePreferenceString:(NSString *)aStr forKey:(NSString *)aKey;
- (NSString *)databasePreferenceStringForKey:(NSString *)aKey;

#pragma mark Change Notifications
- (void)disableNotifications;
- (void)enableNotifications;
- (BOOL)areNotificationsEnabled;


#pragma mark Trash
- (NSArray*)allTrashedTrashableObjectsForUser: (GWUser*)aUser includeOwnedAndDelegatedBy: (BOOL)includeOwnedAndDelegated;


#pragma mark Ports
@property (nonatomic,readonly) NSInteger accessdPort;
@property (nonatomic,readonly) NSInteger attachmentdPort;
@property (nonatomic,readonly) NSInteger DAVPort;
@property (nonatomic,readonly) NSInteger DAVNoSSLPort;
@property (nonatomic,readonly) NSInteger calPublishDPort;


#pragma mark LVH
@property (nonatomic, readonly) BOOL isHosted;

@end
