//
//  DLQuickSearchQueryItemSearchSpec.h
//  DLViews
//
//  Created by Fraser Kuyvenhoven on 2012.07.19.
//  Copyright (c) 2012 Marketcircle Inc. All rights reserved.
//

#import <Foundation/Foundation.h>

@class BDQualifier, BDSortOrdering;


typedef enum
{
	DLQueryItemSearchSpecPriorityHigh,
	DLQueryItemSearchSpecPriorityMedium,
	DLQueryItemSearchSpecPriorityLow
} DLQueryItemSearchSpecPriority;

typedef enum
{
	DLQueryItemSearchSpecTypeFullText,
	DLQueryItemSearchSpecTypeFullTextPrefix,
	DLQueryItemSearchSpecTypePhone,
	DLQueryItemSearchSpecTypeElectronicAddress,
	DLQueryItemSearchSpecTypeGeoAddress,
	DLQueryItemSearchSpecTypeFirstLastName,
	DLQueryItemSearchSpecTypeForm,
	DLQueryItemSearchSpecTypeSubquery
} DLQueryItemSearchSpecType;


@interface DLQuickSearchQueryItemSearchSpec : NSObject

@property (nonatomic) DLQueryItemSearchSpecType searchType;
@property (nonatomic) DLQueryItemSearchSpecPriority priority;
@property (nonatomic, strong) NSArray *sortOrderings;
@property (nonatomic, strong) BDQualifier *additionalQualifier;
@property (nonatomic, strong) NSArray *nonDisplayingSearchKeys; // search keys that should not be displayed
@property (nonatomic, strong) NSArray *displayEntireValueSearchKeys; // search keys that should show entire value (not excerpt)

+ (DLQuickSearchQueryItemSearchSpec *)searchSpecificationWithType:(DLQueryItemSearchSpecType)aSearchType searchKeysAndDisplayNames:(NSArray *)aSearchKeysAndDisplayNames;
+ (DLQuickSearchQueryItemSearchSpec *)searchSpecificationWithType:(DLQueryItemSearchSpecType)aSearchType key:(NSString *)aKey displayName:(NSString *)aDisplayName;

+ (DLQuickSearchQueryItemSearchSpec *)searchSpecificationForFullTextWithKey:(NSString *)aKey displayName:(NSString *)aDisplayName;
+ (DLQuickSearchQueryItemSearchSpec *)searchSpecificationForFullTextPrefixWithKey:(NSString *)aKey displayName:(NSString *)aDisplayName;
+ (DLQuickSearchQueryItemSearchSpec *)searchSpecificationForFullTextWithKeysAndDisplayNames:(NSArray *)aSearchKeysAndDisplayNames;
+ (DLQuickSearchQueryItemSearchSpec *)searchSpecificationForFullTextPrefixWithKeysAndDisplayNames:(NSArray *)aSearchKeysAndDisplayNames;

#pragma mark Init

- (id)initWithSearchType:(DLQueryItemSearchSpecType)aSearchType searchKeysAndDisplayNames:(NSArray *)aSearchKeysAndDisplayNames;

- (BDQualifier *)qualifierWithSearchString:(NSString *)aSearchString;

@property (nonatomic, readonly, strong) NSArray *searchKeys;
- (NSString *)displayNameForSearchKey:(NSString *)aSearchKey;

@end
