//
//  GWAttachmentCache.h
//  GWSubstrate
//
//  Created by Sebastien Boisvert on 2010-05-28 21.
//  Copyright 2010 Marketcircle, Inc. All rights reserved.
//

#import "MCFileCache.h"
#import "DLPObjectContext.h"
#import "GWFileAttachment.h"

typedef enum {
	kGWServerFileCacheType = 0,
	kGWUserFileCacheType,
	kGWDMAFileCacheType // same as kGWUserFileCacheType, but used to force a proper path initialization
} GWFileCacheType;

#if TARGET_OS_IPHONE

#define CacheErrorLog(_format_, _args_...) \
do { \
	MCLog(_cmd, self, _format_, ## _args_); \
} while (0)

#else

#define CacheErrorLog(_format_, _args_...) \
do { \
	if(kGWDMAFileCacheType == [[GWAttachmentCache sharedCache] type] && [[[NSProcessInfo processInfo] environment] objectForKey:@"APP_SANDBOX_CONTAINER_ID"]) \
	{ \
		dispatch_once(&DMAWarnOnce, ^{ \
			MCLog(_cmd, self, _format_, ## _args_); \
			NSLog(@"This non-fatal warning is being reported because you are likely running a sandboxed version of Mail. This will be the only warning reported this session."); \
		}); \
	} \
	else \
		MCLog(_cmd, self, _format_, ## _args_); \
} while (0)

#endif


@interface GWAttachmentCache : MCFileCache {
	GWFileCacheType type;
}


@property (readwrite) GWFileCacheType type;

+(GWAttachmentCache*)sharedCache;
+(GWAttachmentCache*)setupSharedFileCacheOfType:(GWFileCacheType)type;

-(NSString*)relativeFilePathForObject:(GWFileAttachment*)attachment;

@end
