//
//  GWRemindableObjectProtocol.h
//  GWSubstrate
//
//  Created by Sebastien Boisvert on Tuesday 26/08/08.
//  Copyright 2008 Marketcircle Inc. All rights reserved.
//

#import "GWSubstrateDefines.h"
#import "GWUser.h"
#import "GWReminder.h"

#import "BDKeyValueQualifier.h"
#import "BDAndQualifier.h"

@protocol GWRemindableObject

/**
*  Returns all the reminders (including snooze reminders) of the object for the given user.
*
*  @param user the GWUser object to retrieve the reminders for
*
*  @return an NSArray of 0 or more GWReminder objects.
*/
-(NSArray*)allRemindersForUser:(GWUser*)user;

/**
*  Returns all the reminders (including snooze reminders) of the object for the  current object context user.
*
*  @return an NSArray of 0 or more GWReminder objects.
*/
-(NSArray*)allRemindersForCurrentUser;

/**
*  Returns all the non-snooze reminders of the object for the given user.
*
*  @param user the GWUser object to retrieve the reminders for
*
*  @return an NSArray of 0 or more GWReminder objects
*/
-(NSArray*)remindersForUser:(GWUser*)user;

/**
*  Returns all the non-snooze reminders of the object for the given user.
*
*  @return an NSArray of 0 or more GWReminder objects
*/
-(NSArray*)remindersForCurrentUser;

/**
*  Get a reminder to use for snoozing the object.
*
*  @discussion Will attempt to find an already fired snooze reminder for re-use (to limit creation of snooze reminders for the object), otherwise a new snooze reminder is created and returned.
*
*  @return a GWReminder object configured as a snooze reminder; it is up to the caller to configure it for the proper trigger date & time (unit, measurement and/or datetime as appropriate)
*/
-(GWReminder*)snoozeReminderForCurrentUser;

/**
*  Determine if the object has any active reminders for any user.
*
*  @return YES if there are any active reminders, otherwise NO.
*/
@property (nonatomic,readonly) BOOL hasReminder;

/**
*  The active reminders, for any user.
*
*  @return an NSArray of 0 or more GWReminderObjects.
*/
@property (nonatomic,readonly) NSArray *activeReminders;

/**
*  All reminder of type GWReminderDayliteDisplayType for any user.
*
*  @return an NSArray of 0 or more GWReminderObjects.
*/
@property (nonatomic,readonly) NSArray *displayRemindersForCurrentUser;

/**
*  All reminder of type GWReminderDayliteEmailType for any user.
*
*  @return an NSArray of 0 or more GWReminderObjects.
*/
@property (nonatomic,readonly) NSArray *emailRemindersForCurrentUser;

/**
*  All reminder of type soundRemindersForCurrentUser for any user.
*
*  @return an NSArray of 0 or more GWReminderObjects.
*/
@property (nonatomic,readonly) NSArray *soundRemindersForCurrentUser;

/**
*  The default reminder for the current object context user (legacy).
*
*  @return an NSArray of 0 or more GWReminderObjects.
*/
@property (nonatomic,readonly) GWReminder *defaultReminderForCurrentUser;

- (GWReminder *)defaultReminderForUser:(GWUser *)aUser;

@end
