// GWReminder.h
// GWSubstrate
//
// Copyright (c) 2001 Marketcircle Inc. All rights reserved.


#import "_GWReminder.h"

extern NSString*       GWReminderSettingsMeasurement;
extern NSString*       GWReminderSettingsUnits;

@class MCEnumWrapper;

@interface GWReminder : _GWReminder
{
id	referencedObject;
id 	timer;

}

@property(weak) MCEnumWrapper *typeEnumWrapper;
@property(weak) MCEnumWrapper *unitEnumWrapper;

@property(readonly) NSArray *applicableUnitEnumWrappers;

@property(nonatomic,weak) NSMutableDictionary *reminderSettingsDictionary;
@property(nonatomic,weak) NSString *note;
@property(nonatomic,weak) NSString *emailAddress;
@property(nonatomic,weak) NSString *soundName;
@property(nonatomic,weak) NSString *runScriptName;
@property(nonatomic,weak) NSString *openFileName;

#pragma mark Reminders
@property(readonly) NSString *reminderFullDescription;
@property(readonly) NSString *reminderTypeString;
@property(readonly) NSString *reminderDescriptionString;
@property(readonly) NSDate *calculateFireDateTimeAndReturn;

+ (NSString *)reminderDescriptionForUnit:(NSNumber *)u measurement:(NSNumber *)m;

+ (NSNumber*)unitsInMinutesWithUnits:(NSInteger)aUnit measurement:(NSInteger)aMeasurement;

#pragma mark Reminder Creation
+ (void)createNewRemindersFromDefaultsForUser:(GWUser *)aUser;
// convenience method to create a user's default reminders for a specific appt/task
+ (NSArray *)createDefaultRemindersForUser:(GWUser *)aUser forObject:(id)anObject;
+ (void)calculateFireDateTimeForInviteeReminders:(id)reminders;

+ (NSDate *)eventDatetimeForReminder:(GWReminder *)aReminder withFireDatetime:(NSDate *)fireDatetime;

-(NSString *)reminderDescription;

- (id)parentObject;
- (void)smartSetObject:(id)anObj;

- (void)setReferencedObject:(NSObject *)aValue;
- (NSObject *)referencedObject;

- (void)setTimer:(NSTimer *)aValue;
- (NSTimer *)timer;

- (NSString *)unitString;

- (BOOL)isActive;
- (void)setIsActive:(BOOL)flag;

// returns timeblock or task
- (id)object;

- (NSDate *)objectDate;

// Invalidates the timer and if active call the didDeleteReminder notification
- (void)invalidate;

//INvalidates the time only
- (void)invalidateTimer;

-(NSString *)iCalRepresentation;
- (NSString *)iCalRepresentationForObject:(id)anObj OfType:(NSString *)type emailAddress:(NSString *)email soundName:(NSString *)sound;

- (NSDate *)getFireDateTimeReminderForTimeblockWithStartDate:(NSDate*)startDate;

- (NSDate*)fireDateOnly;
- (void)setFireDateOnly:(NSDate*)aDate;

- (NSDate*)fireTimeOnly;
- (void)setFireTimeOnly:(NSDate*)aTime;

-(void)configureReminderFromDictionary:(NSDictionary *)settings;
-(NSDictionary *)reminderRepresentationDictionary;

-(void)clearFiredDatetime;

-(NSInteger)reminderDelta;
-(NSDate*)eventDatetimeForReminder;
- (NSNumber*)unitsInMinutes;
- (NSNumber*)unitsInMinutesWithUnits:(NSInteger)aUnit measurement:(NSInteger)aMeasurement;
- (void)calculateFireDateTime;
-(NSDate*)calculateNextOccurrenceDate;
- (NSDate*)calculateFireDateTimeWithUnits:(NSNumber*)aUnit
measurement:(NSNumber*)aMeasurement;

-(NSDate*)rawFireDate;

-(BOOL)isSnooze;

/**
*  If the reminder has fired or not.
*
*  @return YES if the reminder has fired, NO if it is due to fire.
*/
-(BOOL)hasFired;


-(NSString*)generateReminderAlertIDForDate:(NSDate*)anEventDate;

#pragma mark Default Reminders
#if TARGET_OS_IPHONE
+ (void)createDefaultRemindersIfNecessaryForCurrentUserWithObject:(id)anObj;
+ (GWReminder *)createReminderForObject:(id)anObject linkToObject:(BOOL)flag;
#endif

#pragma mark EnumWrappers
+ (NSArray *)typeEnumWrappers;
+ (NSArray *)unitEnumWrappers;
- (NSArray *)applicableUnitEnumWrappers;

#pragma mark ReminderSettingsDictionary
- (void)setReminderSettingWithKey:(NSString *)aKey value:(id)anObj;
- (id)reminderSettingForKey:(NSString *)aKey;

#pragma mark Snooze Support
+ (void)snoozeObject:(id)object withDate:(NSDate*)origDate onDateTime:(NSDate*)aDateTime withMeasurement:(NSNumber *)measurement unit:(NSNumber *)unit;
+ (void)snoozeObject:(id)object withDate:(NSDate*)origDate onDateTime: (NSDate*)aDateTime;
+ (void)snoozeObject:(id)object withMeasurement:(NSNumber *)measurement unit:(NSNumber *)unit;
+ (void)snoozeObject:(id)object withDate:(id)aDate measurement:(NSNumber *)measurement unit:(NSNumber *)unit;
+ (void)snoozeObject:(id)object withDate:(NSDate*)origDate forTimeInterval:(NSTimeInterval)aTimeInterval;
+ (void)snoozeObject:(id)object forTimeInterval:(NSTimeInterval)aTimeInterval;
+ (void)snoozeObject:(id)object untilDateTime:(NSDate*)aDateTime;

@end

