//
//  GWSourceList.h
//  GWSubstrate
//
//  Created by Sebastien Boisvert on 2010-08-31.
//  Copyright 2010 Marketcircle, Inc. All rights reserved.
//

/*
 
 Minimalist implementation of the GWSourceList class; taken from the old implementation now called GWSourceList_old; if you need look at the old code, look there.
 
 If you need to re-enable the functionality not yet ported from the old class for testing (while still using this current implementation), simply change this class to be a subclass of GWSourceList_old
 
 */

extern NSString *kGWSourceListAddedObjectsNotification;
extern NSString *kGWSourceListRemovedObjectsNotification;
extern NSString *kGWSourceListChangedObjectsKey;

#import "_GWSourceList.h"
//#import <MCFoundation/MCSyncDefines.h>
//#import <MCFoundation/MCSourceListDefines.h>


#ifndef MCSourceListTypeDef
typedef NS_ENUM(NSInteger, MCSourceListType) {
    MCSourceListTypeSystem				= 10,
    MCSourceListTypeSystemRecents       = 20,
    MCSourceListTypeTaskInbox           = 5,
    MCSourceListTypeTaskWorklist        = 6,
    MCSourceListTypeTaskUnfiled         = 7,
    MCSourceListTypeSync                = 22,
    MCSourceListTypeTouch               = 23,
    MCSourceListTypeApplicationCustom   = 25,
    MCSourceListTypeWatch               = 30,
    MCSourceListTypeSmart               = 40,
    MCSourceListTypeDumb                = 50
};

#define MCSourceListTypeDef
#endif

#ifndef MCSourceListVisibilityDef
typedef NS_ENUM(NSInteger, MCSourceListVisibility) {
    MCSourceListVisibilityPrivate				= 0,
    MCSourceListVisibilityShared				= 1
};

#define MCSourceListVisibilityDef
#endif


@class GWTask,GWGroup;
@class BDQualifier;
@class DLSearchCriteria;

typedef NS_ENUM(NSInteger, GWSourceListApplicationType)
{
    GWSourceListApplicationTypeCalendarCategories = 10,
    GWSourceListApplicationTypeCalendarUsers = 20,
    
    // Special Task System List Types - sourcelists will  not be marked as MCSourceListTypeApplicationCustom typeCode - this is a bit of a hack
    GWSourceListApplicationTypeTaskDueToday = 100,
    GWSourceListApplicationTypeTaskInProgress = 110,
    GWSourceListApplicationTypeTaskNotDone = 120,
    GWSourceListApplicationTypeTaskSomeday = 130,
    GWSourceListApplicationTypeTaskDelegatedToMe = 140,
    GWSourceListApplicationTypeTaskDelegatedByMe = 150,
    GWSourceListApplicationTypeTaskDone = 160,
};

@interface GWSourceList : _GWSourceList
{
}

+ (int)destinationTypeCodeForEntityNamed: (NSString*)entityName;

- (void)dumbListAddObjects: (id)objects;
//- (void)dumbListAddObject: (id)object;
//- (void)dumbListRemoveObject: (id)object;
- (void)dumbListRemoveObjects: (id)object;
//- (NSArray*)results;
- (BDQualifier*)qualifier;
- (NSString*)entityName;
- (void)setOwner: (id)anOwner;
- (id)owner;
- (BOOL)isDefault;
- (MCSourceListType)type;
- (void)setType: (MCSourceListType)aType;
- (void)setOrder: (NSInteger)order;
- (void)markAsDefault;
- (void)setEntityName: (NSString*)anEntityName;
- (NSNumber*)primaryKey;
-(BOOL)isDumbList;

@property (strong) DLSearchCriteria *criteria;
@property (strong) NSDictionary *criteriaPlist;

@property(nonatomic,readwrite,assign) BOOL isHidden;

//- (void)willDisplayResults;
//- (void)didDisplayResults;
//- (BOOL)containsObject: (id)object;

- (GWSourceListApplicationType)applicationType;
- (void)setAppliationType: (GWSourceListApplicationType)type;

#pragma mark Calendar Publishing

@property (readonly) BOOL supportsCalendarPublishing;
@property (nonatomic, assign) BOOL publishCalendar;

#pragma mark Display State

// Returns nil if not found.
#if !TARGET_OS_IPHONE
//- (GWDisplayState*)findDisplayStateForUser:(GWUser*)user;

// Creates and returns a new display state if not found.
//- (GWDisplayState*)displayStateStorageObjectForUser:(GWUser*)user;
#endif
@end
