//
//  GWTask+RepeatManagerAdditions.h
//  DLSubstrate
//
//  Created by Patrick Rogers MC on 13-11-01.
//  Copyright (c) 2013 Marketcircle Inc. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "GWTask.h"
#import "DLRepeatingObject.h"
#import "GWSubstrateDefines.h"

typedef enum {
    DLRepeatingTaskCompletionPromptAnswerCancel,            // The user elected not to modify the task
    DLRepeatingTaskCompletionPromptAnswerNotRepeating,      // The task is not repeating and the instance can be marked done
    DLRepeatingTaskCompletionPromptAnswerThisOccurrence,    // A repeat exception should be created for the selected object and only it should be marked done
    DLRepeatingTaskCompletionPromptAnswerAllPrevious        // The selected instance and all previous instances should be marked complete
} DLRepeatingTaskCompletionPromptAnswer;

typedef DLRepeatingTaskCompletionPromptAnswer (^DLTaskStatusEditingPromptCallback)(NSString *title,NSString *detail,NSString *singlePrompt,NSString *allPrompt);

@interface GWTask (RepeatManagerAdditions) <DLRepeatingObject>
/**
 * Prompts the user to determine how they want to edit self when updating task status. If self is a repeating persistent
 * object then the callback will be passed an appropriate query to determine how the object should be edited. The query
 * will take into account what kind of occurrence this is (first, typical, last) to only present the user with valid choices.
 * This method can be used to prompt the user before editing or after editing is complete.
 *
 * @param statusCode The statusCode that will be assigned to the task.
 * @param promptCallback The callback block that will be used to perform the actual query. The promptCallback will be
 *                       called with localized query strings and should return the choice that was selected. Any strings
 *                       that are nil are not valid choices in the current context and can be ignored.
 *
 * @return The type of exception needed to update the status of self.
 */
- (DLRepeatingTaskCompletionPromptAnswer)promptForSettingTaskStatus:(GWTaskStatus)statusCode promptCallback:(DLTaskStatusEditingPromptCallback)promptCallback;

/**
 * Updates the status of a repeating task. It will prompt the user if necessary about what type of exception they would
 * like to generate using the prompt callback and it will create one or more exception objects to edit if necessary or
 * dictated by the prompt answer. If self is a repeat occurrence proxy then an exception will always be generated
 * (since proxies cannot be edited), if self is persistent repeating object then an exception will be generated
 * based on the answer supplied by the user. If the user elects to update all previous instances of the task before
 * the current occurrence (self) then an exception will be created for each instance up to and including self.
 *
 * @param statusCode The status code to assign to the task
 * @param objectContextIn The object context in which to create the editable object.
 * @param prompt The callback block to use to pose the question. See promptForEdigingRepeat: for more details.
 * @param answer A previously supplied answer about what type of exception to create for this edit.
 *
 * @return The exception object that is now a stand in for self or nil if the prompt answer indicated that the user cancelled
 *         the operation. This may be self if self is not repeating. If self was previously selected in a view then the
 *         selection should be updated to the return value since it is now the proxy for the previously selected object.
 *         The returned exception may be used to edit the task further, but it is probably best to call instanceForEditingInObjectContext:queryCallback:
 *         if the intention is to edit properties other than the task status since this method may create additional exceptions
 *         that will not be returned.
 */
- (GWTask *)setTaskStatus:(GWTaskStatus)statusCode objectContext:(MCPObjectContext *)objectContextIn queryCallback:(DLTaskStatusEditingPromptCallback)prompt;
- (GWTask *)setTaskStatus:(GWTaskStatus)statusCode objectContext:(MCPObjectContext *)objectContextIn promptAnswer:(DLRepeatingTaskCompletionPromptAnswer)answer;
@end
