// GWTask.h
// GWSubstrate
//
// Copyright (c) 2001 Marketcircle Inc. All rights reserved.


#import "_GWTask.h"
#import "DLAlertableObjectProtocol.h"
#import "DLDelegatableObject.h"

@class GWCalendar;
@class GWProject;
@class MCEnumWrapper;

@interface GWTask : _GWTask <DLAlertableObjectProtocol,DLDelegatableObject>
{
    NSDate *oldDueDate; // if the user sets someday we clear out the due date but we cache it here in case they uncheck someday again
    BOOL wasSomeday;
    
@private
    BOOL __didCreate;
    BOOL _didCompleteTask;	// used to determine if we need to create next repeat upon save
    BOOL deferRepeatRuleUpdates;    // This is used by the repeat manager but needs to be defined here
    
}

@property(assign,getter=isDone) BOOL done;

@property(readonly) NSArray *reverseDelegateChainEntries;
@property(readonly) NSArray *allLinkedMajorObjects;

// Returns YES if there are any entries in the delegation chain delegated by the current user and marked for notifying
@property(assign) BOOL notifyCurrentUserOfDelegationChanges;

@property (nonatomic,assign) NSString *name;
@property (nonatomic,readonly) NSArray *allLinkedObjects;

@property (nonatomic,readonly) NSString *statusString;
@property (nonatomic,readonly) NSString *dueDateString;
@property (nonatomic,readonly) NSString *shortDueDateString;
@property (nonatomic,readonly) NSString *mediumDueDateString;
@property (nonatomic,readonly) id lastObjective;

@property (nonatomic) BOOL hasStartDate;
@property (nonatomic) BOOL hasCompleteDate;
@property (nonatomic) BOOL hasDueDate;
@property (nonatomic,readonly) NSNumber *sortablePriority;

@property (nonatomic,readonly) NSNumber *durationSeconds;

// Indirection here so we can share this code between GWTask and DLTaskOccurrenceProxy objects, both of whom will call into this method.
+ (NSString *)dueDateStringForTask:(GWTask *)aTask;

- (void)populateAsFollowUpTaskForObject:(id)obj;


-(GWDelegateChainEntry*)lastDelegateChainEntryRelevantToUser:(GWUser*)aUser;

- (NSString*)statusStringForCode: (NSNumber*)code;
- (id)dueDateFallsWithin:(NSDate *)aDate :(NSDate *)anotherDate;
- (id)dueDateFallsOn:(NSDate *)aDate;
- (id)belongsToCategory:(id)aQual;
- (id)hasStatus:(id)aQual;
- (id)lockOnDates:(NSArray *)dates;
- (id)hasPriority:(id)qual;

/**
 Only changes the status code of the task, and does not trigger any of the side effects of calling -setStatusCode: does (like moving from lists or affecting subtasks, etc.)
 */
- (void)setStatusCodeOnly:(NSNumber *)aStatusCode;

// This happens when the status code is set to done
- (void)setSubtasksParentTaskDoneIfApplicable;

-(void)setParentChainNotDoneAndAddToUnfiledIfNeeded;

- (void)setSubtasksCancelled;

- (void)inheritLinksFromParentTask:(GWTask*)parent;

- (void)linkObjectToSubtaskHeirarchy:(id)anObj;
- (void)unlinkObjectFromSubtaskHeirarchy:(id)anObj;

///// Breaks out the start date, start time, and duration (end date/time) for easier UI bindings

// Takes the current startDate, resets its date values (year, month, day) and then sets the resulting NSDate
// as the new startDate
// Also does the same for the end date,i.e. adjusting the end date to be the same date as the startDate
// NOTE: the getter just returns startDate, i.e. it does not remove the time
@property (nonatomic,retain) NSDate *startDateOnly;

// Takes the current startDate, resets its time values (timezone, hour, minute, second) and then sets the resulting
// NSDate as the new startDate
// This method also adjusts the endDate so that the duration between the two is maintained.
// NOTE: the getter just returns startDate, i.e. it does not remove the date
@property (nonatomic,retain) NSDate *startTimeOnly;

- (BOOL)validateStartDate:(id*)ioValue error:(NSError**)outError;
- (BOOL)validateCompleteDate:(id*)ioValue error:(NSError**)outError;

@property (nonatomic,retain) NSDate *completeDateOnly;
@property (nonatomic,retain) NSDate *completeTimeOnly;

- (NSString *)displayString;
- (NSString *)displayStringForLength:(NSUInteger)aLength;

//- (void)setFireDateTimeForReminder:(NSDate *)date; unneeded?

//- (void)cascadeUnlinkAndClearFromCalendar:(id)aCal; unneeded?

@property (nonatomic,readonly) NSString *priorityString;

@property (nonatomic,readonly) BOOL isOverdue;
@property (nonatomic,assign) BOOL isOnWorklist;

//#if TARGET_OS_IPHONE
//@property (nonatomic) BOOL addToInboxOnSave;
//#endif

- (NSArray*)allObjectives;

- (NSString *)allContactsAsString;
#if !TARGET_OS_IPHONE
- (NSString *)allGroupsAsString;
#endif
- (NSString *)allProjectsAsString;
- (NSString *)allOppsAsString;
- (NSString *)allObjectivesAsString;
- (NSString *)allOrganizationsAsString;
- (NSString *)keywordsString;

- (NSString *)descriptionForGlobalFind;
- (NSString *)descriptionForGroupsList;

@property (nonatomic,readonly) BOOL hasParentTask;
- (GWTask *)rootTask;

- (BOOL)doesParentChainContainTask: (GWTask *)task;

// anObject must have a selector -tasks. Returns the first uncompleted task, nil if there aren't any
+ (GWTask *)nextPendingTaskForObject:(id)anObjectThatHasTasks;
+ (GWTask *)lastTaskForObject:(id)objectThatHasTasks;
+ (NSArray *)pendingTasksForObject:(id)objectThatHasTasks;
+ (NSArray *)sortedTasksForObject:(id)objectThatHasTasks;
+ (NSInteger)pendingTasksCountForObject:(id)obj;

//+ (BOOL)moveTask:(GWTask *)aSub toNewParent:(id)newParent; unneeded?

+ (BOOL)canMoveTask:(GWTask *)aTask up:(BOOL)moveUp;
+ (BOOL)canOutdentTask:(GWTask *)aTask;
+ (BOOL)canIndentTask:(GWTask *)aTask;

+ (BOOL)canIndentTasks:(NSArray *)tasks;
+ (BOOL)canOutdentTasks:(NSArray *)tasks;

+ (BOOL)indentTask:(GWTask *)task;
+ (BOOL)outdentTask:(GWTask *)task;

+ (BOOL)indentTasks:(NSArray *)tasks;
+ (BOOL)outdentTasks:(NSArray *)tasks;

// given tasks or subtasks, makes sure that unique top level tasks are returned only (put into the array)
+ (NSUInteger)rootTasksForTasks:(NSArray *)tasks placeInto:(NSMutableArray *)rootedTasks;

- (NSString *)iCalRepresentation;
- (NSNumber *)durationMinutes;
- (NSNumber *)durationSeconds;
- (NSString*)durationDaysHoursMin;
#if !TARGET_OS_IPHONE
- (NSDecimalNumber *)billingAmount;
- (NSArray *)billableObjects;
#endif
- (NSArray *)allTasks;
- (NSArray *)tasksInSubTaskHeirarchyMatchingQualifier:(BDQualifier *)qual;
- (void)addSubTasksToArray:(NSMutableArray *)array;
#if !TARGET_OS_IPHONE
- (BOOL)isBillable;
#endif
- (void)markAsInProgress;
- (BOOL)isCancelled;
- (BOOL)isDoneOrCancelled;

-(BOOL)hasOpenSubtasks;

// works up the heirarchy of inheretance for the receiver and if any in the chain are in the array, returns YES
- (BOOL)inParentalChainOfTasks:(NSArray *)tasks;

// works through the array, for each task checking if any are the child or in the child chain
- (NSArray *)tasksInChildrenChainOfTasks:(NSArray *)tasks;

// if the passed task is a descendent of the receiver, returns YES
- (BOOL)hasDescendentTask:(GWTask *)task;

// PORTME
//-(void)takeFirstTimeValuesFromPropertyList:(NSDictionary *)values currentDocument:(id)doc;
//-(void)takeUpdateValuesFromPropertyList:(NSDictionary *)values currentDocument:(id)doc;

- (BOOL)hasSubtasks;

#if !TARGET_OS_IPHONE
- (NSString *)fullEstimatedTimeString;
#endif

- (NSNumber *)smartEstimatedTime;
- (void)recalculateCachedTotalEstimatedTime;
- (void)triggerCachedTotalEstimatedTimeRecalculation;

- (GWProject *)convertToProjectAndDelete:(BOOL)shouldDelete;

- (void)updateETag;

#pragma mark -
#pragma mark Accessors
- (NSDate *)oldDueDate;
- (void)setOldDueDate:(NSDate *)anOldDueDate;

- (BOOL)isSomeday;
- (void)setIsSomeday:(BOOL)flag;

- (void)setKeywords:(NSArray *)keywords;

#pragma mark -
#pragma mark Inbox Support
- (BOOL)isInInbox;
- (void)addToInboxForUser:(GWUser*)user;
- (void)removeFromInbox;

#pragma mark -
#pragma mark Unfiled Support
- (BOOL)isInUnfiled;
- (void)removeFromUnfiled;

#pragma mark -
#pragma mark Worklist Support
-(void)removeFromWorklist;
-(void)removeFromWorklistIfNeeded;

// this will assign the task to the given user, removing the task from the current owner's inbox/unfiled in the process
// if changing owner, this should be called *before* the owner is changed so that it gets properly removed from current owner's list
- (void)addToUnfiledForUser:(GWUser*)aUser;

#pragma mark -
#pragma mark Locations
// PORTME
//- (void)setLocation:(NSString *)aString;

#pragma mark -
#pragma mark Source Lists
- (DLPObject *)sourceListJoinInDumpList: (GWSourceList*)aSourceList;
- (NSUInteger)orderOfTaskInDumbList: (GWSourceList*)aSourceList;

#pragma mark -
#pragma mark Enum Wrappers
+ (NSArray *)statusEnumWrappers;
@property(nonatomic,strong) MCEnumWrapper *statusEnumWrapper;

+ (NSArray *)typeEnumWrappers;
@property(nonatomic,strong) MCEnumWrapper *typeEnumWrapper;

+ (NSArray *)priorityEnumWrappers;
+ (MCEnumWrapper*)noPriorityEnumWrapper;
@property(nonatomic,strong) MCEnumWrapper *priorityEnumWrapper;

@end
