//
//  GWTimeblock+Meetings.h
//  GWSubstrate
//
//  Created by Patrick Rogers MC on 10-03-23.
//  Copyright 2010 Marketcircle, Inc. All rights reserved.
//

#import "GWTimeblock.h"

@interface GWTimeblock (Meetings)
@property (nonatomic,readonly) NSString *commaSeparatedResourcesList;
@property (nonatomic,readonly) BOOL hasInvitees;
@property (nonatomic,readonly) NSArray *unsentMeetingInvites;
@property (nonatomic,readonly) NSArray *pendingMeetingInvites;
@property (nonatomic,readonly) NSArray *acceptedMeetingInvites;
@property (nonatomic,readonly) NSArray *rejectedMeetingInvites;
@property (nonatomic,readonly) NSArray *pendingInvitees;
@property (nonatomic,readonly) NSArray *acceptedInvitees;
@property (nonatomic,readonly) NSArray *rejectedInvitees;
@property (nonatomic,readonly) NSArray *meetingInvitees;
@property (nonatomic,readonly) NSArray *meetingInviteesAsContacts;
@property (nonatomic,readonly) NSArray *sortedMeetingInvitees;
@property (nonatomic,readonly) NSArray *meetingAttendees;
@property (nonatomic,readonly) NSArray *sortedMeetingAttendees;

// If all attendees have responded (i.e. there's no pending or unsent invites)
@property(nonatomic,readonly) BOOL haveAllAttendeesResponded;

- (BOOL)isUserInvited:(GWUser *)aUser;
- (BOOL)hasUserAccepted:(GWUser *)aUser;
- (BOOL)hasUserRejected:(GWUser *)aUser;
- (BOOL)isUserPending:(GWUser *)aUser;

- (BOOL)isVisibleToCalendarOwner:(id)aCalendarOwner;
- (BOOL)isUserMeetingInvitee:(GWUser *)aUser;
- (BOOL)isUserPendingOrAcceptedMeetingInvitee:(GWUser *)aUser;
- (GWMeetingInvite *)meetingInviteForUser:(GWUser *)aUser;
- (GWMeetingInvite *)meetingInviteForContact:(GWContact *)contact;

// This method will look at the unsaved changes and record any that might need to have a notification sent out.
- (void)recordUnsavedChangesForMeetingUpdates;

/**
* Adds an invite for the user and marks it as new (Used mainly for the MeetingManager when it
* needs to deal with changes to the invitee list - when a new user is added from the CreateEdit
* Timeblock card it gets marked as new, and then the MeetingManager uses that to know to switch
* it to pending and send out the invitation alert
*/
- (GWMeetingInvite *)addInviteForUser:(GWUser *)aUser;
- (GWMeetingInvite *)addInviteForContact:(GWContact *)contact;
- (void)addInvitees:(NSArray *)invitees;

- (void)removeMeetingInvite:(GWMeetingInvite *)invite;
- (void)removeInvitees:(NSArray *)invitees;
- (void)removeInvitee:(GWUser *)invitee;
+ (void)removeInvitee:(GWUser *)invitee fromMeetings:(NSArray *)meetings;

// These methods will add an invite for the user if one does not already exist
- (void)acceptInviteForUser:(GWUser *)aUser;
- (void)acceptInvitationForInvitee:(GWUser *)invitee;
- (void)acceptInvitationForInvitee:(GWUser *)invitee withNote:(NSString *)note;
- (void)rejectInvitationForInvitee:(GWUser *)invitee;
- (void)rejectInvitationForInvitee:(GWUser *)invitee withNote:(NSString *)note;
+ (void)invitee:(GWUser *)invitee rejectsAttendanceToMeetings:(NSArray *)meetings;
- (void)pendInviteForUser:(GWUser *)aUser;

- (void)finalizeMeetingInvites;

- (void)notifyInviteesAboutMeetingChanges:(NSDictionary *)changes withNote:(NSString *)note;
- (void)notifyInviteesAboutMeetingDeletion;
- (void)rescheduleMeetingForChanges:(NSDictionary *)changes withNote:(NSString *)note;

- (NSArray*)unsentMeetingInvites;
- (NSArray*)pendingMeetingInvites;
- (NSArray*)acceptedMeetingInvites;
- (NSArray*)rejectedMeetingInvites;

@end

// This method should implemented above DLSubstrate.
@interface GWTimeblock (AppKitExternalMeetingInvites)
- (void)sendiMIPUsingDefaultEmailerSendIndividualUpdates:(BOOL)individualUpdates;
@end

