//
// GWTimeblock.h
// DTSubstrate
//
// Copyright (c) 2001 Marketcircle Inc. All rights reserved.

#import "_GWTimeblock.h"
#import "DLAlertableObjectProtocol.h"
#import "GWRemindableObjectProtocol.h"
#import "DLDelegatableObject.h"

typedef enum _MCTimeblockType {
    MCTimeblockNoType				= 0,
    MCTimeblockAppointmentType		= 1,
    MCTimeblockEventType			= 2,
    MCTimeblockTaskRelated			= 3,
    MCTimeblockOutboundCall			= 4,
    MCTimeblockInboundCall			= 5,
    
    MCTimeblockChat                 = 6,
    MCTimeblockHoliday              = 7,
    MCTimeblockMeeting              = 8,
    MCTimeblockVideo                = 9
} MCTimeblockType;

typedef enum _GWTimeblockType {
    GWTimeblockTypeAppointment = 1,
    GWTimeblockTypeEvent = 2,
    GWTimeblockTypeOutboundCall = 4,
    GWTimeblockTypeInboundCall = 5,
    GWTimeblockTypeChat = 6,
    GWTimeblockTypeVacation = 7,
    GWTimeblockTypeMeeting = 8,
    GWTimeblockTypeVideo = 9
} GWTimeblockType;

typedef enum _GWTimeblockStatus {
    GWTimeblockStatusOther     = 0,
    GWTimeblockStatusTentative = 1,
    GWTimeblockStatusConfirmed = 2,
    GWTimeblockStatusPostponed = 3,
    GWTimeblockStatusCompleted = 4,
    GWTimeblockStatusCancelled = 5,
    GWTimeblockStatusPencilIn  = 6
} GWTimeblockStatus;

typedef enum {
    kGWTimeblockMeetingInviteePending  = 0,
    kGWTimeblockMeetingInviteeAccepted = 1,
    kGWTimeblockMeetingInviteeRejected = 2
} GWTimeblockMeetingInviteeStatus;

@class GWTimeblockOccurrenceProxy;
@class GWUser;
@class GWMeetingInvite;
@class MCEnumWrapper;

@interface GWTimeblock : _GWTimeblock <DLAlertableObjectProtocol,DLDelegatableObject>  {
    NSInteger startDateAsSecondOfDay;
    NSInteger visibilityCache;
    BOOL deferRepeatRuleUpdates;
    
    NSDictionary*   meetingUpdateChanges; // used to cache changed attributes which will trigger a meeting update or reschedule
}
@property(nonatomic,readwrite,strong) GWUser *assignedTo;
@property(readonly) NSArray *reverseDelegateChainEntries;
// Returns YES if there are any entries in the delegation chain delegated by the current user and marked for notifying
@property(assign) BOOL notifyCurrentUserOfDelegationChanges;

@property (nonatomic,strong) NSString *name;
@property (nonatomic,readonly) NSString *displayStringForDates;
@property (nonatomic,readonly) NSString *twoLineDateRangeDescription;
@property (nonatomic,readonly) NSString *oneLineDateRangeDescription;
@property (nonatomic,readonly) NSString *typeString;
@property (nonatomic,copy) NSNumber *durationDays;
@property (nonatomic,copy) NSNumber *durationMinutes;
@property (nonatomic) NSTimeInterval durationSecondsValue;
@property (nonatomic,copy) NSNumber *durationSeconds;
@property (nonatomic, readwrite, copy) NSNumber* smartDurationSeconds;
@property (nonatomic,strong) NSDictionary *meetingUpdateChanges;
@property (nonatomic,copy) MCEnumWrapper *statusEnumWrapper;
@property (nonatomic,copy) MCEnumWrapper *typeEnumWrapper;
@property (nonatomic,readonly) BOOL shouldDisplayAsBannerEvent;
@property (nonatomic,readonly) BOOL shouldDisplayAsMultiDayBannerEvent;

// If a meeting invite exists for the current user, this will return it.
@property (nonatomic,readonly) GWMeetingInvite *currentUserMeetingInvite;

/**
 * Settting the startDate of a timeblock will not affect the endDate.
 * setting startDatePreservingDuration will update the startDate and
 * maintain the same duration for the timeblock.
 */
@property (nonatomic,copy) NSDate *startDatePreservingDuration;
/**
 * Setting the timezone of a timeblock will store the timezone name so that it
 * can be archived. Changing the timezone will not change the time range of a
 * timeblock unless it is set to nil which will change the timeblock to floating
 * in which case the startDate and endDate will always be remapped to be at the
 * same time of day in the current timezone.
 */
@property (nonatomic,copy) NSTimeZone *timeZone;
/**
 * Setting the timeZone property of a timeblock doesn't change the start and end times.
 * The timeZonePreservingTime property will change the timezone but preserve the time
 * of the timeblock in the new timezone by setting it to floating before setting the
 * timezone.
 */
@property (nonatomic,copy) NSTimeZone *timeZonePreservingTime;

@property (nonatomic,readonly) BOOL isAllDay;
@property (nonatomic) BOOL floating;
@property (nonatomic,retain) NSDate *rawStartDate;
@property (nonatomic,retain) NSDate *rawEndDate;
@property (nonatomic,retain) NSDate *rawRepeatEndDate;

+ (NSTimeInterval)bannerPromoteMinDuration;

#pragma mark -
#pragma mark Timeblock Convenience Accesors

+ (GWTimeblock *)lastTimeblockForObject:(id)objectThatHasTimeblocks;
+ (GWTimeblock *)nextPendingTimeblockForObject:(id)objectThatHasTimeblocks;
+ (NSArray *)pendingTimeblocksForObject:(id)objectThatHasTimeblocks;
+ (NSArray *)sortedTimeblocksForObject:(id)objectThatHasTimeblocks;
+ (NSInteger)pendingTimeblocksCountForObject:(id)obj;

#pragma mark -
#pragma mark Display Strings
/**
 * These methods provide the implementation for the corresponding property accesors
 * with the same name so that GWTimeblockOccurrenceProxy can share the implmentation.  There
 * probably isn't much of a good reason to call these directly.
 *
 * @param timeblock The timeblock to investigate.  Should be an instance of GWTimeblock
 *                  or GWTimeblockOccurrenceProxy.
 */
+ (NSString *)twoLineDateRangeDescription:(id)timeblock;
+ (NSString *)twoLineDateRangeDescriptionWithStartDate:(NSDate *)startDate endDate:(NSDate *)endDate timeZone:(NSTimeZone *)timeZone allDay:(BOOL)isAllDay showTimeZone:(BOOL)showTimeZone;

+ (NSString *)displayStringForDates:(id)timeblock;

// 'Fri, 29 Mar at 11:00 AM to 1:15 PM'
+ (NSString *)oneLineDateRangeDescriptionForStartDate:(NSDate *)aStartDate endDate:(NSDate *)anEndDate isAllDay:(BOOL)allDay useRelativeDay:(BOOL)relative;
+ (NSString *)oneLineDateRangeDescriptionForStartDate:(NSDate *)aStartDate endDate:(NSDate *)anEndDate isAllDay:(BOOL)allDay; // passes NO for useRelativeDay

- (void)updateETag;

#pragma mark -
#pragma mark Enum Wrappers

+ (NSArray *)statusEnumWrappers;
- (NSArray *)statusEnumWrappers;

+ (NSArray *)typeEnumWrappers;
- (NSArray *)typeEnumWrappers;

#pragma mark -
#pragma mark Conveniences

- (void)populateAsFollowUpTimeblockForObject:(id)obj;
- (NSString *)statusString;
- (NSString *)keywordsString;

@property (nonatomic,readonly) BOOL isObjectProxyEvent;
/**
 * Returns a status code to use when redering this timeblock in a calendar for a set of users.
 * If users contains a single user then the returned status code is that users status as a meeting
 * invitee. If users contains multiple values then the returned value is the status code of the
 * meeting. If self is a proxy timeblock that wraps another object or is not visible for the current
 * user then the return value is GWTimeblockStatusOther.
 *
 * @param users An array of users that define the calendar that is being displayed.
 * @return The status code to use to display the timeblock in the calendar.
 */
- (GWTimeblockStatus)renderStatusForUsers:(NSArray *)users;

#pragma mark -
#pragma mark Date/Time

+ (NSDate *)defaultEndDateForStartDate:(NSDate *)startDate objectContext:(DLPObjectContext *)objectContext;
- (void)setEndDateFromStartDateWithDuration:(NSInteger)minutes;
- (void)setEndDateFromStartDateUsingDefaultDuration;

#pragma mark -
#pragma mark Overlap
- (BOOL)overlapsTimeblock:(GWTimeblock*)otherTimeblock;

#pragma mark -
#pragma mark Suppressing Automatic ETag updating
- (void)suppressETagDuringBlock:(void(^)(void))updateBlock;

@end

@interface NSArray(GWTimeblockFilters)
- (NSArray *)filteredTimeblocksWithStartDate:(NSDate *)startDate endDate:(NSDate *)endDate;
- (NSArray *)filteredTimeblocksWithTypeCodeEqualTo:(NSInteger)typeCode;
- (NSArray *)filteredTimeblocksWithTypeCodeNotEqualTo:(NSInteger)typeCode;
@end

