//
//  MCDetailEditAbstractView.h
//  DetailEdit
//
//  Created by Ali Lalani on 09-10-29.
//  Copyright 2009 Marketcircle, Inc. All rights reserved.
//

#import <Cocoa/Cocoa.h>

#import "MCDetailEditDefines.h"

@class MCDetailEditAbstractSubview,MCDetailEditView,MCPObjectContext;
@interface MCDetailEditAbstractView : NSView {
    id model;
    BOOL editing;
}

@property(nonatomic,strong) id model;
@property(nonatomic,assign) BOOL editing;
@property(nonatomic,strong) MCPObjectContext *objectContext;

@property(strong) NSString *titleString;
@property(strong) NSString *titleStringKeyPath;
@property(readonly) NSString *title; // this should, based on the title string or title string key path, return the appropriate item title

@property(nonatomic,strong) NSDictionary *configuration;
@property(nonatomic,strong) NSString *itemIdentifier;

@property(nonatomic,readonly) MCDetailEditView *detailEditView;

@property(strong, nonatomic,readonly) NSMutableArray *items;
@property(nonatomic) BOOL needsLayout;

+ (Class)subItemClass;


#pragma mark -
#pragma mark Init
+ (MCDetailEditAbstractView *)detailEditItemForConfiguration:(NSDictionary *)aConfiguration;

// Because weak refs are gone during the dealloc of an object, we have a separate mechanism to prepare all the items in the heirarchy for release so they can remove their bindings and observations and such.
- (void)detailEditPrepareForRelease;

#pragma mark Conveniences
- (MCDetailEditAbstractView *)firstItemInHeirarchyRespondingToSelector:(SEL)aSelector;


#pragma mark -
#pragma mark Items Management
- (void)createItems; // method that creates the items, typically only called when the configuration is set, though a dynamic parent could do this at other times as well.
- (void)createItemsFromConfigurations:(NSArray *)configs;

- (void)propagateModelToSubItems;
- (void)modelWillChange;
- (void)modelDidChange;
- (void)editingDidChange;
- (void)modelOrEditingDidChange;

- (void)addItem:(MCDetailEditAbstractView *)item atIndex:(NSInteger)anIndex;
- (void)addItem:(MCDetailEditAbstractView *)item;
- (void)removeItem:(MCDetailEditAbstractView *)item;
- (void)removeAllItems;

- (NSArray *)hiddenInEditModeItems;
- (MCDetailEditAbstractView *)subItemForItemIdentifier:(NSString *)anIdentifier;
- (void)hasValueKeyPathDidChangeForItem:(MCDetailEditAbstractView *)anItem;

#pragma mark -
#pragma mark Layout
- (BOOL)evaluateItemsForInclusion;
- (void)prepareItemsForLayout;
- (void)layoutSubviews;
- (void)subViewHeightDidChange:(MCDetailEditAbstractSubview *)aSubView;

#pragma mark -
#pragma mark KeyViewCalculation
- (id)initialKeyView;
- (void)recalculateKeyViewLoop;

#pragma mark Localization
// Based on where the configuration was loaded from, this call will look in that bundle for this string with the table name of the plist name that was loaded.
// i.e. DLViews has the detail/edit configuration file DetailEditConfiguration.plist. So this call will look in DLViews, and the strings should be in a file DetailEditConfiguration.strings in that bundle.
- (NSString *)localizedStringFromConfigurationBundleForString:(NSString *)aString;

@end
