//
//  MCDynamicGlobals.h
//  MCMerge
//
//  Created by AJ Dev on 12-09-16.
//
//

#import <Foundation/Foundation.h>

/*!
 This object is meant to be used in the scripting environment of reports. One huge pain the butt with reports is that there is no way to store global variables somewhere. Say you derive some stuff at the top of the report then you want to use it down below. Before you would have to type some elaborate method to achieve this. No longer.
 Now you can simply do the following:
 globals setCounter:1
 globals counter
 
 The above calls are the equivalent of globals setValue:1 forKey:'counter' and globals objectForKey:'counter'.
 
 It just makes the script a bit more tidy.
 
 The only supported pattern is set<var name> and <var name> - the typical get/set pattern. It will throw an exception if you try to go outside those bounds.
 
 Keys (either via setObject:forKey: or setValue:forKey:) cannot have punctuation or whitespace characters in them. An exception will be raised if it encounters them.
 
 
 We could and probably should move this class to MCFoundation since it can be used in multiple places.
 
 */

@interface MCDynamicGlobals : NSObject
@property (retain) NSMutableDictionary *storage;

- (id)objectForKey:(id)aKey;
- (void)setObject:(id)obj forKey:(id)aKey;

- (NSArray *)allKeys;
- (NSArray *)allValues;

// the primitive that stores the value. Subclasses can override to do their own thing and call super if necessary.
- (void)setStorageValue:(id)val forKey:(NSString *)aKey;
@end
