//
//  MCFileCache.h
//  MCFoundation
//
//  Created by Sebastien Boisvert on 2010-05-28 21.
//  Copyright 2010 Marketcircle, Inc. All rights reserved.
//

#if ! TARGET_OS_IPHONE
@class SCEvent, SCEvents, MCFSEventsNotification;
@protocol SCEventListenerProtocol

/**
 * Conforming objects' implementation of this method will be called whenever an
 * event occurs. The instance of SCEvents which received the event and the event
 * itself are passed as parameters.
 */
- (void)pathWatcher:(SCEvents *)pathWatcher eventOccurred:(SCEvent *)event;

@end
#endif

extern NSString *kMCFileCachePathKey;
extern NSString *kMCFileCacheSizeKey;
extern NSString *kMCFileCacheLastAccessKey;
extern NSString *kMCFileCacheLastModifiedKey;

@interface MCFileCache : NSObject
#if ! TARGET_OS_IPHONE
	<SCEventListenerProtocol>
#endif
{

	@private
	NSMutableArray *_cacheData;
	NSURL *_baseCacheDirURL;
	NSUInteger _cacheSize;
	BOOL _useCacheInfoFile;
}

@property (nonatomic,strong) NSMutableArray *_cacheData;
@property (nonatomic,strong) NSURL *_baseCacheDirURL;
@property (assign) NSUInteger _cacheSize;
@property (assign) BOOL _useCacheInfoFile;

#pragma mark -
#pragma mark Subclass Methods
-(NSURL*)createCacheFileForObject:(id)object;
-(NSURL*)cachedPathForObject:(id)object;
-(BOOL)removeCachedPathForObject:(id)object;

#pragma mark -
#pragma mark Class Methods
-(id)initWithPath:(NSURL*)path subPaths:(NSArray*)dirNames useCacheInfoFile:(BOOL)useFile error:(NSError **)error;
-(id)initWithApplicationName:(NSString*)appName subPaths:(NSArray*)dirNames inDomain:(NSSearchPathDomainMask)domain  useCacheInfoFile:(BOOL)useFile error:(NSError **)error;
-(void)setMaxFileCacheSize:(NSUInteger)size;
-(NSURL*)absoluteFilePathForObject:(id)object;

#if ! TARGET_OS_IPHONE
- (void)pathWatcher:(SCEvents *)pathWatcher eventOccurred:(SCEvent *)event;
#else
-(void)pathChanged:(NSURL*)eventURL;
#endif

@end
