
#import <Foundation/Foundation.h>

// DataStore Types - used when creating a MCPDataStoreInfo instance
typedef NS_ENUM(NSInteger, MCPDataStoreType) {
    kMCPDataStorePostgreSQLType = 1,
    kMCPDataStoreSQLiteType,
    kMCPDataStoreMySQLType,
    kMCPDataStoreTokyoCabinetType,
    kMCPDataStoreHTTPType
};


// Controls the save behaviour of ObjectContexts
typedef NS_ENUM(NSInteger, MCPObjectContextSaveMode) {
    kMCPObjectContextSaveAutoMode = 0,
    kMCPObjectContextSaveManualMode,
    kMCPObjectContextSaveImmediateMode
};


typedef NS_ENUM(NSInteger, MCPDatabaseType) {
	kMCPStandardDatabaseType = 0,
	kMCPSampleDatabaseType = 1,
	kMCPOfflineDatabaseType = 2,
    kMCPMirrorDatabaseType = 3,
	kMCPHostedDatabaseType = 4,
    kMCPTouchDatabaseType = 5
};


typedef enum {
    kMCPCountAggregateType = 1,
    kMCPSumAggregateType,
    kMCPMaximumAggregateType,
    kMCPMinimumAggregateType,
    kMCPAverageAggregateType,
    kMCPNowAggregateType // This is a special case, it just returns the curent datetime from the server
} MCPAggregateType;



@class MCPAsyncFetchRequest;
typedef void (^MCPAsyncFetchRequestCallback)(MCPAsyncFetchRequest* request, NSArray* results, BOOL wasCancelled, NSError* error);

typedef void (^MCPAsyncResultCountCallback)(MCPAsyncFetchRequest* request, NSInteger resultCount, BOOL wasCancelled, NSError* error);

// Async fetching block signature
typedef void (^MCPAsyncFetchResultsBlock)(NSArray* results, NSError* error);
typedef void (^MCPAsyncResultCountBlock)(NSUInteger count, NSError* error);



#pragma mark Sorting Defines
typedef NS_ENUM(NSInteger, MCPSortOrderType) {
    kMCPSortOrderingNormalType = 0
};


// Debugging Related
#ifdef MCDEBUG

// If this define is turned on the adaptors will record each query that is executed along with statistics about each query
#define MCPERSISTENCE_DEBUG_QUERY_HISTORY 0

#define MCPERSISTENCE_MCPOBJECT_EXCEPTION_ON_ERROR 1

#endif


// Controls whether we will support auto updating the in-memory object graph
#define MCPERSISTENCE_ENABLE_AUTO_UPDATES 1
#if MCPERSISTENCE_ENABLE_AUTO_UPDATES && 0
#define MCPAutoUpdateLog(_format_, _args_...) \
do { \
NSLog(_format_, ## _args_); \
} while (0)
#else
#define MCPAutoUpdateLog(_format_, _args_...)
#endif




extern NSString* MCPersistenceExceptionName;


#pragma mark -
#pragma mark Error Handling
#pragma mark -

// There are varying levels of error reporting built into the persistence layer.  This is because various levels of the persistence layer can be used independently of the others.  For example, the DataConnection level could be used directly, or the Adaptor level could be used directly, etc.
// To facilitate this there is an error domain for each level in the persistence layer.  Starting from the top where most people work (the error code range in brackets, the levels are:  ObjectContext (100), DataStore (200), Coordination (300), Adaptor (400), DataConnection (500), specific data store implementation (i.e. PG, SQLite, etc.) (varying codes as these are generally the data store specific error codes wrapped in an NSError - could conflict with the codes used for the persistence layer).
// Because each level builds upon itself in the persistence layer, lower level errors are wrapped in higher level errors.  For example, if there is an SQL error related to fetching data there will likely be 3 levels of NSError's wrapped.  The caller asking for the fetch from the ObjectContext would get an NSError from the OC level of codes (100), with an underlying NSError from the DataConnection level (500), with an underlying NSError from the data store specific error (i.e. PG).
// When an NSError is logged from the persistence layer the tree of underlying errors are included in the output.

extern NSString*  kMCPersistenceObjectContextErrorDomain;
extern NSString*  kMCPersistenceDataStoreErrorDomain;
extern NSString*  kMCPersistenceCoordinationErrorDomain;
extern NSString*  kMCPersistenceAdaptorErrorDomain;
extern NSString*  kMCPersistenceDataConnectionErrorDomain;


// Error Codes
typedef NS_ENUM(NSInteger, MCPersistenceError) {
	
	kMCPUnknownError = 1,
    
    // ObjectContext Error Codes
    
    
    // DataStore Error Codes
    
    
    // Coordination Error Codes
    kMCPCoordinationMainAdaptorConnectionError = 300,
    kMCPCoordinationUpdateAdaptorConnectionError = 301,
    kMCPCoordinationPoolCloseAdaptorError = 310,
    kMCPCoordinationPoolAdaptorConnectionError = 311,
    
    // Adaptor Error Codes
    //kMCPAdaptor Error = 40,
    kMCPAdaptorPrimaryKeyCreationError = 410,
    kMCPAdaptorFaultRelationshipError = 411,
    kMCPAdaptorFaultOrderedManyToManyError = 412,
    kMCPAdaptorMassDeleteError = 413,
    kMCPAdaptorMassUpdateError = 414,
    kMCPAdaptorAutoUpdatingStartError = 415,
    kMCPAdaptorAutoUpdatingStopError = 416,
    
    kMCPAdaptorConnectionValuesStorageError = 420,
    kMCPAdaptorConnectionValuesClearError = 421,
    kMCPAdaptorConnectionValuesUpdateError = 422,
    
    kMCPAdaptorPersistAddOperationError = 430,
    kMCPAdaptorPersistUpdateOperationError = 431,
    kMCPAdaptorPersistUpdateOperationMissingPkeyError = 432,
    kMCPAdaptorPersistDeleteOperationError = 433,
    kMCPAdaptorPersistDeleteOperationMissingPkeyError = 434,
    kMCPAdaptorPersistDeleteOperationMissingPkeyAttributeError = 435,
    kMCPAdaptorPersistDeleteJoinOperationError = 436,
    
    kMCPAdaptorTransientCreationError = 440,
    kMCPAdaptorTransientRulePopulationError = 441,
    kMCPAdaptorTransientDestroyError = 442,
    kMCPAdaptorTransientFetchRelatedError = 443,

	// DataConnection Errors
    kMCPDataConnectionConnectionError = 500,
    kMCPDataConnectionQueryError = 501,
    kMCPDataConnectionRawRowQueryError = 502,
    kMCPDataConnectionRawRowInsertError = 503,
    kMCPDataConnectionRawRowInsertNoValuesError = 504,
    kMCPDataConnectionRawRowUpdateError = 505,
    kMCPDataConnectionRawRowUpdateNoValuesError = 506,
    kMCPDataConnectionRawRowDeleteError = 507,
    kMCPDataConnectionRawRowDeleteNoValuesError = 508,
    kMCPDataConnectionBindingParseError = 509,
    kMCPDataConnectionTransactionBeginError = 510,
    kMCPDataConnectionTransactionCommitError = 511,
    kMCPDataConnectionTransactionRollbackError = 512,
    
    kMCPDataConnectionUsageError = 800
	
};



#pragma mark Notification Names/Keys

extern NSString*  kMCPDidSaveNotification;
// User Info keys for DidSaveNotification
extern NSString*  kMCPChangeDescriptionUserInfoKey;

#pragma mark ConnectionValue Keys
extern NSString* kMCPUserIDConnectionValueKey;
extern NSString* kMCPClientIDConnectionValueKey;


typedef NS_ENUM(NSInteger, MCPSupportedPlatforms) {
    MCPSupportedPlatformsAll          = 0,
    MCPSupportedPlatformsDesktopOnly = 1,
    MCPSupportedPlatformsTouchOnly   = 2,
    MCPSupportedPlatformsCustomOnly  = 3
};

typedef NS_ENUM(NSInteger, MCPDeleteRule) {
	MCPDeleteRuleNullify 	= 0,
	MCPDeleteRuleCascade	= 1,
	MCPDeleteRuleDeny		= 2,
	MCPDeleteRuleNoAction	= 3
};




/******
 Brought over from MCFoundation when more code was being copied wholesale, but then that code was removed because it got problematic.
 The stuff below needs to be vetted to see if the concept still make sense.
 
typedef enum _MCPPrimaryKeyState {
	MCPPrimaryKeyNotDetermined 	= 0,
	MCPPrimaryKeyNotSet			= 1,
	MCPPartialPrimaryKey		= 2,
	MCPValidPrimaryKey			= 3
} MCPPrimaryKeyState;



//An object is unlocked by default. An editor locks it before editing and then unlock it after it's done. In the event of deferred saving, the lock state of an editing object should be set to unlock after a save is done. In the event save is called while the editing still have the object selected, then it shouldn't unlocked after saving.


typedef enum _MCPObjectLockState {
	MCPUnLocked 			= 0,
	MCPLocked				= 1,
	MCPUnlockAfterSave		= 2
} MCPObjectLockState;


typedef enum _MCPDeleteRule {
	MCPDeleteRuleNullify 	= 0,
	MCPDeleteRuleCascade	= 1,
	MCPDeleteRuleDeny		= 2,
	MCPDeleteRuleNoAction	= 3
} MCPDeleteRule;



typedef enum _MCPDatabaseQueryType {
	kMCPDatabaseSelectQueryType = 0,
	kMCPDatabaseCountQueryType,
	kMCPDatabaseRelationshipFaultQueryType	
} MCPDatabaseQueryType;


extern NSString *MCPMoreThanOneException;
extern NSString *MCPValidationException;


// Sent from the object context when an object has been added or removed from another object via a keyed relationship (as defined in the entity and model) -- Useful for updating list views ...
// The notification's object is the objectContext, the userInfo contains the keys described below
extern NSString *MCPWillAddRelatedObjectNotification;
extern NSString *MCPDidAddRelatedObjectNotification;
extern NSString *MCPWillRemoveRelatedObjectNotification;
extern NSString *MCPDidRemoveRelatedObjectNotification;

// Related Object Changed Notification Keys
extern NSString *MCPDocumentKey;
extern NSString *MCPParentObject;
extern NSString *MCPRelatedObject;
extern NSString *MCPRelationshipKey;

extern NSString *MCPWillChangeDatabaseConnectionNotification;
extern NSString *MCPDidChangeDatabaseConnectionNotification;

**/

@class MCPDefines;
@interface MCPDefines : NSObject

+ (MCPDefines*)objectForLocalization;

@end
