//
//  MCPFetchResult.h
//  MCFoundation
//
//  Created by Ali Lalani on 10-03-16.
//  Copyright 2010 Marketcircle, Inc. All rights reserved.
//

@class MCPFetchSpecification,DLPObjectContext,MCPFetchResultsPlaceholderProxy,MCPObject,BDQualifier,MCPTransientFetchSession;

typedef enum
{
	MCPFetchResultsControllerFetchStateInitial,
	MCPFetchResultsControllerFetchStateFetching,
	MCPFetchResultsControllerFetchStateRefreshing,
	MCPFetchResultsControllerFetchStateSelect,
	MCPFetchResultsControllerFetchStateDone
} MCPFetchResultsControllerFetchState;


@interface MCPFetchResultsController : NSObject {
    MCPFetchSpecification *fetchSpecification;
    /* used for "temporary" filtering, so as not to disturb the original fetch spec
     This will be AND'd with the fetch spec qualifier */
    BDQualifier *additionalQualifier;
    
    NSMutableArray *results; // the publicly available array of results
    NSMutableArray *silentUpdateResults; // used when we're doing a silent update
    BOOL silentUpdate;
    BOOL allResultsLoaded; // have all the results been loaded, or are there still batches remaining?
    
	// Updating
	BOOL autoUpdates; // listens to change notifs and re-fetches when necessary
	
    // Selection
    NSIndexSet *selectionIndexes;
    NSNumber *initialPrimaryKeySelection; // once the list has finished loading, select the object with this primary key first.
	
    // Batching
    Class placeholderProxyClass; // if you want your placeholder objects in batching mode to be a custom class

@private
    NSUInteger currentQueryBatchNumber;
    NSMutableArray *queryQueue;
    NSMutableArray *unqueuedBatches; // unfetched and unqueued batches, i.e. how much is left to do
    NSString *batchQueryTempTableIdentifier;
	MCPTransientFetchSession *fetchSession;
    MCPFetchSpecification *finalFetchSpecification;
	BOOL batchFetchingInitialized; // This is set to YES in batch mode when the transient storage has been fully set up.
    BOOL isFetching;
    BOOL needsRefetch;
}

@property(nonatomic,readonly,strong) MCPFetchSpecification *finalFetchSpecification; // returns self.fetchSpec or a new one with the additionalQualifier if applicable
@property(nonatomic,readonly, strong) MCPFetchSpecification *fetchSpecification;
@property(readonly) NSString *entityName;
@property(nonatomic,strong) BDQualifier *additionalQualifier;
@property(nonatomic,strong) NSString *fetchResultsControllerIdentifier; // this can optionally be set to make debugging easier, i.e. to differentiate this instance from other ones.

@property(nonatomic,weak) DLPObjectContext *objectContext;
@property(nonatomic,strong,readonly) MCPTransientFetchSession *fetchSession;
@property(weak) id delegate;

@property(nonatomic,assign) BOOL autoUpdates;
// people observing selection index changes should ignore changes if this is set to YES
@property(nonatomic,assign) BOOL ignoreSelectionIndexChanges;

@property(assign, readonly) MCPFetchResultsControllerFetchState fetchState;
@property(assign) BOOL batchFetchingInitialized;
@property(nonatomic, readonly, strong) NSMutableArray *results;
@property(readonly) BOOL allResultsLoaded;

@property (nonatomic, assign) BOOL fetchingSuspended;

// Selection
@property(nonatomic,strong) NSIndexSet *selectionIndexes;
@property(nonatomic,readonly) NSArray *selectedObjects; // derived, based on selection indexes

@property(assign) Class placeholderProxyClass;

- (id)initWithObjectContext:(DLPObjectContext *)anObjectContext fetchSpecification:(MCPFetchSpecification *)aFetchSpecification;

#pragma mark -
#pragma mark Fetching
- (void)fetchResults;
// Triggers a fetch of all results, not asynchronously, even if batching is turned on.
- (void)fetchAllResultsBlocking;

#pragma mark -
#pragma mark Fetch Spec
- (void)updateFinalFetchSpecification;

#pragma mark -
#pragma mark Batching Support
- (void)proxyObjectDidAct:(MCPFetchResultsPlaceholderProxy *)aProxyObject;
- (void)selectObject:(MCPObject *)anObject;
- (void)selectObjectsWithPrimaryKeys:(NSArray *)primaryKeys;
//- (void)selectObjectWithPrimaryKey:(NSNumber *)aPrimaryKey;

// Calls a block (on the main thread) when it has successfully loaded the batch that contains the object for the primary key provided.
- (void)loadObjectWithPrimaryKey:(NSNumber *)aPrimaryKey notify:(void (^)(id loadedObject))aBlock;
@end

@protocol MCPFetchResultsControllerDelegate <NSObject>
@optional
- (void)fetchResultsControllerWillChangeResults:(MCPFetchResultsController *)fetchResult;
- (void)fetchResultsControllerDidChangeResults:(MCPFetchResultsController *)fetchResult;
- (void)fetchResultsController:(MCPFetchResultsController *)fetchResult willReplaceProxiesAtIndexes:(NSIndexSet *)indexes withObjects:(NSArray *)objs;
- (void)fetchResultsController:(MCPFetchResultsController *)fetchResult didReplaceProxiesAtIndexes:(NSIndexSet *)indexes withObjects:(NSArray *)objs;

- (void)fetchResultsController:(MCPFetchResultsController *)fetchResult initialPrimaryKeySelectionForSilentRefresh:(id*)objectRef;

// If delegates want to do any post processing on the objects before we return to the main thread
- (void)fetchResultsController:(MCPFetchResultsController *)aFetchResultsController didFetchBatchObjects:(NSArray *)objs;
@end

