//
//  MCPObjectContext.h
//  MCPersistence
//
//  Created by Michael Clark on 10-03-21.
//  Copyright 2010 Marketcircle, Inc. All rights reserved.
//

#include "MCPDefines.h"





extern NSString*  kMCPObjectContextCreatedNotification; // posted when an object context is initialized
extern NSString*  kMCPObjectContextDestroyedNotification; // posted when an object context is prepared for release




@class MCPDataStore;
@class MCPFetchRequest,MCPAggregateRequest,MCPAsyncFetchRequest,MCPAsyncRawSQLRequest,MCPGroupByRequest,MCPAsyncGroupByRequest;
@class MCPFetchSession;
@class MCPTransientFetchSession;
@class MCPFetchSpecification;
@class BDQualifier;
@class MCPWriteQueue;
@class MCPObject;
@class MCPModel;
@class MCPDataStoreInfo;
@class MCUserDefaults;
@class MCPConstraintsManager;
@class MCPGroupByRule;


#if  TARGET_OS_IPHONE
@class MCPTouchClientChangeManager;
#endif

/**
 The main interface into a data store - used for fetching, updating and deleting.
 
 Instances of this class should generally be created using the class method.
 
 */

#import "MCPAutoUpdatingDelegate.h"

@interface MCPObjectContext : NSObject <MCPAutoUpdatingDelegate> {
    
@protected
    MCPDataStore*    dataStore;
    
    MCPWriteQueue*   writeQueue;
    
@private
    MCPObjectContext*  parent;
}
@property (nonatomic,readonly,strong) MCPDataStore*  dataStore;
@property(nonatomic,readonly,strong)MCPWriteQueue* writeQueue;
@property(nonatomic,readonly,strong) MCPConstraintsManager* constraintsManager;

@property(nonatomic,readonly)NSDictionary* mainAdaptorProperties;

@property(nonatomic,readwrite,assign) MCPObjectContextSaveMode saveMode;

@property(nonatomic,readonly,strong)MCPObjectContext*  parent;

@property(nonatomic,strong) MCUserDefaults *userDefaults;
@property(weak, readonly) NSString *userDefaultsPath;
@property(weak, readonly) NSNumber *userDefaultsHostID;
@property(nonatomic,readonly,assign) BOOL isSaving;


@property(nonatomic,readonly) NSNumber* schemaVersion;
@property(nonatomic,readonly) NSString* databaseUUID;
@property(nonatomic,readwrite,weak) NSNumber* primaryKeyOffset;
@property(nonatomic,readwrite,retain) NSDate* trialExpiryDate;

@property(nonatomic,readwrite,assign) MCPDatabaseType databaseType;
@property(nonatomic,readonly) BOOL isSampleDatabase; // convenience

@property(nonatomic,readwrite,assign) BOOL isUpdatingSchema;

@property(nonatomic,readwrite,assign) BOOL isMirrorSyncing;

+ (id)objectContextForDataStoreInfo:(MCPDataStoreInfo *)aDataStoreInfo error:(NSError**)error;

- (id)initWithDataStore: (MCPDataStore*)aStore;

- (void)prepareForRelease;

#pragma mark -
#pragma mark What to call this section
@property(weak, nonatomic,readonly) MCPModel* model;
@property(weak, nonatomic,readonly) MCPDataStoreInfo* dataStoreInfo;
- (NSString*)databaseName;

#pragma mark -
#pragma mark Synchronous Access
- (NSArray*)resultsForFetchRequest: (MCPFetchRequest*)aFetchRequest;

- (NSUInteger)resultCountForFetchRequest: (MCPAggregateRequest*)aRequest;
- (NSUInteger)resultForAggregateRequest: (MCPAggregateRequest*)aRequest;
- (NSDate*)dateResultForAggregateRequest: (MCPAggregateRequest*)aRequest;

- (NSArray*)resultsForGroupByFetchRequest: (MCPGroupByRequest*)aRequest;

- (NSArray*)objectsForFetchSpecification:(MCPFetchSpecification *)aFetchSpec;
- (NSArray*)objectsForEntityNamed: (NSString*)aName;
- (NSArray*)objectsForEntityNamed: (NSString*)aName matchingQualifier: (BDQualifier*)aQualifier;
- (id)objectForEntityNamed: (NSString*)aName wherePrimaryKeyValueIs: (NSNumber*)aValue;


#pragma mark Aggregate Function
- (NSUInteger)resultCountForEntityNamed: (NSString*)aName;
- (NSUInteger)resultCountForEntityNamed: (NSString*)aName matchingQualifier: (BDQualifier*)aQualifier;

- (NSUInteger)resultSumForEntityNamed: (NSString*)aName onAttributeNamed: (NSString*)anAttributeName;
- (NSUInteger)resultSumForEntityNamed: (NSString*)aName onAttributeNamed: (NSString*)anAttributeName matchingQualifier: (BDQualifier*)aQualifier;

- (NSUInteger)resultMinForEntityNamed: (NSString*)aName onAttributeNamed: (NSString*)anAttributeName;
- (NSUInteger)resultMinForEntityNamed: (NSString*)aName onAttributeNamed: (NSString*)anAttributeName matchingQualifier: (BDQualifier*)aQualifier;

- (NSUInteger)resultMaxForEntityNamed: (NSString*)aName onAttributeNamed: (NSString*)anAttributeName;
- (NSUInteger)resultMaxForEntityNamed:(NSString *)aName onAttributeNamed:(NSString *)anAttributeName matchingQualifier: (BDQualifier*)aQualifier;

- (NSUInteger)resultAverageForEntityNamed: (NSString*)aName onAttributeNamed: (NSString*)anAttributeName;
- (NSUInteger)resultAverageForEntityNamed:(NSString *)aName onAttributeNamed:(NSString *)anAttributeName matchingQualifier: (BDQualifier*)aQualifier;

- (NSDate*)minDateForEntityNamed: (NSString*)aName dateAttributeNamed: (NSString*)anAttributeName;
- (NSDate*)minDateForEntityNamed:(NSString *)aName dateAttributeNamed:(NSString *)anAttributeName matchingQualifier: (BDQualifier*)aQualifier;
- (NSDate*)maxDateForEntityNamed: (NSString*)aNAme dateAttributeNamed: (NSString*)anAttributeName;
- (NSDate*)maxDateForEntityNamed:(NSString *)aNAme dateAttributeNamed:(NSString *)anAttributeName matchingQualifier: (BDQualifier*)aQualifier;


#pragma mark -
#pragma mark Group By
- (NSArray*)resultsGroupedUsingRule: (MCPGroupByRule*)aRule withAggregateMappings: (NSArray*)mappings forEntityNamed: (NSString*)anEntityName;
- (NSArray*)resultsGroupedUsingRule: (MCPGroupByRule*)aRule withAggregateMappings: (NSArray*)mappings forEntityNamed: (NSString*)anEntityName matchingQualifier: (BDQualifier*)aQualifier;

- (MCPAsyncGroupByRequest*)resultsGroupedUsingRule: (MCPGroupByRule*)aRule withAggregateMappings: (NSArray*)mappings forEntityNamed: (NSString*)anEntityName matchingQualifier: (BDQualifier*)aQualifier andNotify:(MCPAsyncFetchRequestCallback)block notifyQueue: (dispatch_queue_t)aQueue;






- (NSDate*)currentDateTimeAtDataStore;

// Search the data store directly - doesn't confer with in-memory caches because we want the truth in the data store incase we want to perform operations on the data store directly based on the outcome of this method
- (BOOL)doesObjectExistForEntityNamed: (NSString*)anEntityName matchingPrimaryKey: (NSNumber*)aPrimaryKey;

// These methods go straight to the backing data store and do not update the in-memory object graph in anyway
- (BOOL)deleteObjectsForEntityNamed: (NSString*)aName matchingQualifier: (BDQualifier*)aQualifer;
- (BOOL)updateObjectsForEntityNamed: (NSString*)anEntityName matchingQualifier: (BDQualifier*)aQualifier withKeyValues: (NSDictionary*)keyValues;

- (void)refreshObject: (id)anObject;

#pragma mark -
#pragma mark Asynchronous Access
- (void)asyncResultsForFetchRequest: (MCPAsyncFetchRequest*)aFetchRequest;

- (MCPAsyncFetchRequest*)objectsForFetchSpecification:(MCPFetchSpecification *)aFetchSpec andNotify:(MCPAsyncFetchRequestCallback)block;
- (MCPAsyncFetchRequest*)objectsForEntityNamed: (NSString*)aName andNotify:(MCPAsyncFetchRequestCallback)block;
- (MCPAsyncFetchRequest*)objectsForEntityNamed: (NSString*)aName matchingQualifier: (BDQualifier*)aQualifier andNotify:(MCPAsyncFetchRequestCallback)block;

#pragma mark -
#pragma mark FetchSession Access
- (MCPFetchSession*)fetchSessionForFetchSpecification: (MCPFetchSpecification*)aFetchSpec;


#pragma mark -
#pragma mark Transient Support
- (MCPTransientFetchSession*)fetchSessionForTransientEntityNamed: (NSString*)aName fetchSpecification: (MCPFetchSpecification*)aFetchSpec populatedUsingRules: (NSArray*)rules;

#pragma mark -
#pragma mark Object Graph Management
// Creates a new instance for the given entity name and starts tracking it if the insertFlag is YES.  If insertFlag is no then the object will have to be added to the objectContext manually inorder to persist it
- (id)createObjectForEntityNamed: (NSString*)aName insert: (BOOL)insertFlag;
// has the option to not call setupInitialValues. By default it is yes (in the above method) but rarely you may find that undesirable (i.e. you are duplicating an object from another object)
- (id)createObjectForEntityNamed: (NSString*)aName insert: (BOOL)insertFlag setupInitialValues:(BOOL)initialValuesFlag;

- (void)addObject: (MCPObject*)anObject DEPRECATED_ATTRIBUTE;
- (void)insertObject: (MCPObject*)anObject;
- (void)updateObject: (MCPObject*)anObject;
- (void)deleteObject: (MCPObject*)anObject;
- (void)deleteObject: (MCPObject *)anObject skipTrash: (BOOL)shouldSkipTrash;

- (BOOL)hasUnsavedChanges;
- (BOOL)hasUnsavedChangesSinceDate:(NSDate *)aDate;

- (void)saveChanges;

// Shouldn't be used generally - added to support special clean up for mirror sync
- (void)resetState;

// Separate method for saving sub-object contexts because we want to catch any places where code is calling "saveChanges" and is not in a sub-object context, as this would produce unpredictable and likely undesirable results.
- (void)subObjectContextSaveChanges;


#pragma mark -
#pragma mark Sub ObjectContext
@property(nonatomic,readonly) NSArray* subObjectContexts;
- (instancetype)subObjectContext;
- (void)setupAsSubObjectContext;
- (id)manageObject: (MCPObject*)anObject;
- (id)localObjectForObject: (MCPObject*)anObject;
// For each object, if it is an MCPObject, get it local to this oc. Throws exception if an object can't be fetched locally.
- (NSArray *)localObjectsForObjects:(NSArray *)objs;

#pragma mark -
#pragma mark StringReferenceIdentifier
// Give it a string that you got from calling stringReferenceIdentifier on an MCPObject and it will fetch it from the database.
// Strings are in the form of <EntityName>-<PrimaryKey> i.e. "Contact-1400"
- (MCPObject *)objectForStringReferenceIdentifier:(NSString *)aStringReferenceIdentifier;
- (NSArray *)objectsForStringReferenceIdentifiers:(NSArray *)anArray;


///---------------------------------------------------------------------------------------
/// @name Permissions
///---------------------------------------------------------------------------------------
#pragma mark -
#pragma mark Permissions Support
// This needs to be set before permissions can be enabled
@property(nonatomic,readwrite,weak) NSArray* teamIDsForPermissionsConstraints;
@property(nonatomic,assign) BOOL overridePermissionsSystem; // setting this to YES (as in the case of wanting true "superuser" access) overrides queries and such so they don't get permissions constraints

- (BOOL)doesDataStoreSupportPermissions; // Checks the model to see if any entities support permissions - in future will also verify the datastore has the right columns,tables for permissions support
- (BOOL)isPermissionsSystemEnabled;
- (void)enablePermissionsSystem;
- (void)disablePermissionsSystem;

#pragma mark -
#pragma mark Trash Support
/// Status of the trash system
@property(nonatomic,readonly) BOOL isTrashEnabled;
/// Turn on the trash system
- (void)enableTrashSupport;
/// Turn off the trash system
- (void)disableTrashSupport;

@property(nonatomic,readonly,strong) BDQualifier* isTrashedQualifier;


/// The deletionUserID is set on each object which gets trashed and must be set on the objectContext before calling enableTrashSupport
@property(nonatomic,readwrite,strong) NSNumber* deletionUserID;
/// The deletionIdentifier is set on each object which gets trashed and must be set on the objectContext before calling enableTrashSupport
@property(nonatomic,readwrite,strong) NSString* deletionIdentifier;


- (void)restoreObject: (MCPObject*)anObject;
- (void)restoreObjects: (NSArray*)objects;


- (NSArray*)allTrashedTrashableObjects;
- (NSArray*)allTrashedTrashableObjectsForDeletionUserID: (NSNumber*)aUserID;



///---------------------------------------------------------------------------------------
/// @name Touch Sync
///---------------------------------------------------------------------------------------
#pragma mark -
#pragma mark Touch Sync
@property(nonatomic,readwrite,assign) BOOL isTouchSyncing;
#if  TARGET_OS_IPHONE
@property(nonatomic,readonly,strong) MCPTouchClientChangeManager* changeManager; // For touch sync
@property(nonatomic,readwrite,assign) BOOL isTouchSyncingCancelled;

- (BOOL)applyIncomingChangeSetAndReturnError: (NSError**)error;


#endif



///---------------------------------------------------------------------------------------
/// @name Connection Values
///---------------------------------------------------------------------------------------
#pragma mark -
#pragma mark Connection Values
- (void)setConnectionValue: (id)value forKey: (NSString*)key;

///---------------------------------------------------------------------------------------
/// @name Auto Updating
///---------------------------------------------------------------------------------------
#pragma mark -
#pragma mark Auto Updating
- (BOOL)startMonitoringForAutoUpdatingAndReturnError: (NSError**)error;
- (BOOL)stopMonitoringForAutoUpdating;


///---------------------------------------------------------------------------------------
/// @name Transactions
///---------------------------------------------------------------------------------------
#pragma mark -
#pragma mark Transactions
- (BOOL)startTransactionAndReturnError: (NSError**)error;
- (BOOL)rollbackTransactionAndReturnError: (NSError**)error;
- (BOOL)commitTransactionAndReturnError: (NSError**)error;


@end



@interface MCPObjectContext (RawSQL)

- (NSArray*)rowsForRawSQL: (NSString*)sql error: (NSError**)error;

- (MCPAsyncRawSQLRequest*)rowsForRawSQL: (NSString*)sql andNotify: (MCPAsyncFetchRequestCallback)block;

@end
