//
//  MCPreferencePane.h
//  BIViews
//
//  Created by Alykhan Jetha on 10-03-17.
//  Copyright 2010 Marketcircle, Inc. All rights reserved.
//

//#import <MCApplication/MCNibLoadingView.h>
#import "MCPreferenceButton.h"
#import "MCNibLoadingView.h"
#import "MCDocument.h"

@class MCPreferences;
@class MCPreferencePaneController;
@class DLPObjectContext;

/*!
 Abstract superclass of preference panes.
 */
@interface MCPreferencePane : MCNibLoadingView {
	NSToolbarItem *toolbarItem;
	NSMutableDictionary *toolbarSpec;
	MCPreferenceButton *paletteButton;
    
    __weak id objectContext;
    __weak id doc;
}

@property (strong) NSMutableDictionary *toolbarSpec;
@property (nonatomic, weak) DLPObjectContext *objectContext;
@property (nonatomic, weak) id doc;
@property (nonatomic, assign) BOOL requiresRestart;

// If we're in a sample database, returning YES for this for subclasses means an overlay will be put overtop preventing the user from accessing any of the controls in the preference pane and explaining that they are not available for sample databases.
@property(nonatomic,readonly) BOOL disableForSampleDatabase;
- (void)addSampleDatabaseOverlay; // subclassers can override this if they want to do anything special for the sample database overlay, otherwise super will use the default overlay.

/*!
 Return YES if your view is private for some reason (i.e. view switching stub). Default implementation returns NO.
 */
- (BOOL)isPrivatePane;

/*!
 The item that shows in the toolbar if the pref pane is working in toolbar mode
 */
- (NSToolbarItem *)toolbarItem;
+ (MCPreferenceButton*)button;

/*!
 Information used to display the preference in the toolbar (or palette in the future)
 Currently optional.
 */
+ (NSString *)label;
+ (NSString *)toolTip;
+ (NSString *)paletteLabel;
+ (NSImage *)image;
+ (NSString *)group;
+ (NSArray*)keywords;
+ (NSString *)iconName;
+ (NSString *)classIdentifier;

+ (BOOL)requiresPermissionToModifyPreferencePane;


- (void)selectSubIdentifier:(NSString*)aSubIdentifier;

/*! 
 Mandatory
 The name of the preference. label calls this, the calls localizeString:
 */
- (NSString *)unlocalizedLabel;

/*! 
 Mandatory 
 */


/*! 
 Mandatory 
 */
- (CGFloat)minWidth;

/*! 
 Mandatory 
 */
- (CGFloat)minHeight;

/*
 Implement this if you want to start something *after* the initial animation of this pane in occurs
 */
- (void)paneDidAnimateIn;

/* 
Implement if you would like to do some action before the preference pane is about to be swapped out and dealloced
*/
- (void)paneWillClose:(MCPreferences*)sender;

- (void)preferenceWindowDidResignKey;

- (void)restartDayliteAlertMessage;



@end
