//
//  MCReportWrapper.h
//  MCReporting
//
//  Created by Alykhan Jetha on 11/11/05.
//  Copyright 2005 Marketcircle Inc.. All rights reserved.
//

#import <Cocoa/Cocoa.h>

// >>> begin modifications by iOSXpert >>>

//#import "MCReportingDefines.h"

typedef int MCReportLiveLayoutResizingBehavior;
typedef int MCReportWebActionType;

// <<< begin modifications by iOSXpert <<<


@class MCReportLayout;
@class MCReportFScriptClassLoader;
@class MCReportUserOptions;

@interface MCReportWrapper : NSFileWrapper {
	NSString *originalPath;
	NSDictionary *infoDictionary;
	NSPrintInfo *printInfo;
	MCReportLayout *reportLayout;
	BOOL worksWithMergeKit;
	BOOL isEmbedded;
	
	NSNumber *databaseRepresentationID;
	NSString *databaseRepresentationEntityName;
	
	MCReportFScriptClassLoader *fscriptClassLoader;
}


@property (nonatomic, retain) NSDictionary *infoDictionary;
@property (nonatomic, retain) NSPrintInfo *printInfo;
@property (nonatomic, retain) MCReportLayout *reportLayout;

@property (retain) NSString *originalPath;

/*!
 Set to true if this template is found in the embedded paths
 */
@property (assign) BOOL isEmbedded;

/*!
 In case where the report wrapper's permanent location is a bag of bytes in a database, you can set and use these variables to get to or reference the raw bytes (i.e. BIReportTemplate & 234100)
 */
@property (retain) NSNumber *databaseRepresentationID;
@property (retain) NSString *databaseRepresentationEntityName;

/*!
 Only used in user configurable print templates
 */
@property (retain, readonly) MCReportUserOptions *defaultUserChoices;

/*!
 These methods allow you to put a report wrapper that was originally sourced from a database on disk. This is handy for web reports or web print layouts as embedded resources will require normal file paths to work in WebView;
*/
- (BOOL)isInLocalFileStorage;
- (BOOL)putInLocalFileStorageInDirectory:(NSString *)aPath;
- (BOOL)putOnLocalFileStorage;
- (NSString *)locationInLocalFileStorage;



/*!
 If the original paths match, then returns YES or if the databaseIDs match then returns YES.
 */
- (BOOL)isEqualToReportWrapper:(MCReportWrapper *)aWrapper;

/*!
 If the display name, behavior and type is the same, then returns yes. 
 */
- (BOOL)isLooselyEqualToReportWrapper:(MCReportWrapper *)aWrapper;

- (NSString *)originalPath;

/*!
 Takes the current infoDictionary, printInfo and reportLayout and makes normal filewrappers and replaces the old ones. This is used when saving a template modified by the user in the print window and when writing to file. If you are going to change these ivars and then serialize the wrapper, make sure you call this after the ivars have been set.
 */
- (void)resetFileWrappersWithCurrentContent;


// returns yes if there is a Template.html, Template.rtf, Template.txt and Info.plist
- (BOOL)worksWithMergeKit;
- (BOOL)hasHTMLTemplate;
- (BOOL)hasRTFTemplate;
- (BOOL)hasTXTTemplate;

	// derived from info dictionary - NSStringEncoding. NSMacOSRomanStringEncoding is default
- (NSStringEncoding)mergeStringEncoding;

	// reads the .html or .rtf or .txt data as a string encoded using mergeStringEncoding
- (NSString *)mergeTemplateStringValue;




/*!
 Looks for debug = 1 in the info plist
 */
- (BOOL)debug;


- (NSString *)uniqueURLName;

- (BOOL)worksWithWebView;
- (NSDictionary *)webViewProperties;
- (NSString *)webViewName; // perhaps we should deprecate this
- (NSString *)webViewInitialAction;

/*!
 Loads fscript classes. If debug is yes, then reloads on each call.
 */
- (BOOL)webViewLoadClasses;

- (MCReportWebActionType)webActionTypeForAction:(NSString *)aActionName;

- (Class)webViewActionClassForAction:(NSString *)aActionName;
- (SEL)webViewSelectorForAction:(NSString *)aActionName;
- (NSString *)webViewFilenameForAction:(NSString *)aActionName;

- (NSDictionary *)webViewActionDetailsForName:(NSString *)aActionName;
- (NSString *)webViewClassnameForAction:(NSString *)aActionName;
- (NSString *)webViewSelectorNameForAction:(NSString *)aActionName;

- (NSString *)webViewSharedResourcesPath;

- (NSString *)displayName;
- (NSString *)behavior;
- (NSInteger)specificationVersion;

/*!
 User configuration for special print layouts
*/

- (BOOL)supportsUserConfiguration; // if any of the below return yes, then this returns yes
- (BOOL)supportsUserPrintPageLayout;
- (BOOL)supportsTableConfiguration;
- (BOOL)supportsTableGridConfiguration;
- (BOOL)supportsFontSizeUserChanges;
- (BOOL)supportsUserSavingAsTemplate;

- (MCReportLiveLayoutResizingBehavior)liveResizingBehavior;
- (BOOL)hasResizeScript;
- (NSString *)resizeScript; // script to resize elements after the user had changed the paper size

- (void)removeUserConfigurationKeys;

- (MCReportUserOptions *)defaultUserChoices; // if we are user configurable, we use these values as defaults



- (BOOL)isValidForBehaviors:(NSArray *)behav typeIdentifier:(NSString *)aName;

	// legacy support
- (BOOL)isValidForEntityName:(NSString *)entityName;

@end

@interface NSFileWrapper (MCReportImageSupport)

// returns the next available (unused) name such as img1.jpg, MyImage2.png etc...
// aFilename must be the last path component
// this is slightly better than the default behavior
- (NSString *)nextPreferredResourceNameForFilename:(NSString *)aFilename;

// absorbs the data at path and returns the given embedded name
- (NSString *)takeNextReportImageResourceAtPath:(NSString *)fullpath;

- (NSImage *)imageForReportImageResourceNamed:(NSString *)preferredName;

@end
