//
//  NSArray+MCFiltering.h
//  MCFoundation
//
//  Created by aj on Wed Oct 17 2001.
//  Copyright (c) 2001 __CompanyName__. All rights reserved.
//


/*
 
 Creates a new array with objects that return a non-null or non-zero argument of the passed in selector
 
 Or creates a new array with objects that have been transformed by a block function
 
 */

#import <Foundation/Foundation.h>

typedef BOOL (^MCArrayBlockFilter)(id object);
typedef id (^MCArrayBlockTransform)(id object);

@interface NSArray (MCFiltering)

- (NSArray *)filteredArrayUsingSelector:(SEL)aSel;
- (NSArray *)filteredArrayUsingSelector:(SEL)aSel withArgument:(id)anArg;
- (NSArray *)filteredArrayUsingSelector:(SEL)aSel withArgument:(id)anArg andArgument:(id)secondArg;

- (NSArray *)filteredArrayUsingSelector:(SEL)aSel sortKey:(NSString *)sortKey;

- (NSArray *)mcFilteredArrayWithClass:(Class)filter;
- (NSArray *)mcFilteredArrayUsingBlock:(MCArrayBlockFilter)filterCallback;

//
// Expects an array of MCActionArgumentContainer
- (NSArray *)filteredArrayMatchingAllActionArguments:(NSArray *)anArray;
- (NSArray *)filteredArrayMatchingSomeActionArguments:(NSArray *)anArray;

- (NSArray *)filteredArrayWhereAttributeNamed:(NSString *)aName isLike:(id)anObj;

- (id)objectWithAttribute:(NSString *)attr equalTo:(id)value;

- (NSArray*)filteredArrayWhereStringAttributeName:(NSString*)attrib
                                  isEqualToString:(NSString*)anObj
                           caseInsensitiveCompare:(BOOL)caseFlag;

- (NSArray *)filteredArrayUsingPredicateFormat:(NSString *)aFormat;

#pragma mark - Transforming Arrays

- (NSArray *)mcTransformedArrayUsingBlock:(MCArrayBlockTransform)transformCallback;
@end

@interface NSObject (MCFiltering)

- (id)attributeNamed:(NSString *)aName matches:(id)obj;

- (id)attributeNamed:(NSString *)aName matchesIntegerValue:(NSNumber *)obj;
- (id)attributeNamed:(NSString *)aName matchesLongLongValue:(NSNumber *)obj;


- (NSArray *)filteredArrayOfAttributeName:(NSString *)arrayAttr
                 whereObjectAttributeName:(NSString *)attr matches:(id)obj;

- (NSArray *)filteredArrayOfAttributeName:(NSString *)arrayAttr
                 whereObjectAttributeName:(NSString *)attr matchesIntegerValue:(NSNumber *)obj;

// throw exceptions MCInvalidRelationshipTypeException
// does an isEqual instead of ==
- (NSArray*)filteredArrayOfAttributeName:(NSString *)relKey whereAttributeName:(NSString *)attrib isEqualTo:(id)anObject;
// does case insensitive compare
- (NSArray*)filteredArrayOfAttributeName:(NSString *)relKey whereStringAttributeName:(NSString *)attrib isEqualToString:(NSString *)anObject;

// does localizedCaseInsensitvie compare
- (NSArray*)filteredArrayOfAttributeName:(NSString *)relKey whereStringAttributeName:(NSString *)attrib isEqualToLocalizedString:(NSString *)anObject;
@end
